local LibDeflate = LibStub:GetLibrary("LibDeflate")

function MorthalinRaidTools:OnCommReceived(prefix, message, distribution, sender)
	local senderData = MorthalinRaidTools:GetPlayerInfo(sender);
	if(MorthalinRaidTools.db.global.allowBroadcast and senderData.rankIndex <= MorthalinRaidTools.db.global.broadcastIncomingRank) then
		local decoded;
		if(prefix == "MRTHistory") then
			decoded = LibDeflate:DecompressDeflate(LibDeflate:DecodeForWoWAddonChannel(message));
		else
			decoded = message;
		end
		local success, data = self:Deserialize(decoded);
		if(success) then
			if(sender ~= UnitName("player")) then
				if(prefix == "MRTAddBench") then -- add player on bench
					self:AddBenchFromBroadcast(data, sender);
				elseif(prefix == "MRTRemoveBench") then -- remove player from bench
					self:RemoveBenchFromBroadcast(data, sender);
				elseif(prefix == "MRTFullBench") then -- get whole bench
					for i = 1, #data do
						self:AddBenchFromBroadcast(data[i], sender);
					end
				elseif(prefix == "MRTEmptyBench") then -- remove all from bench
					self:ResetList(true);
				elseif(prefix == "MRTKill") then -- add kill
					local found = false;
					if(GetRaidRosterInfo(1) ~= null) then
						for i = 1, 40 do
							local name, rank, subgroup, level, class = GetRaidRosterInfo(i);
							if(name == sender) then
								found = true;
							end
						end
					end
					if(found == false) then
						self:AddKillFromBroadcast(data, sender);
						MorthalinRaidTools:UpdateHistory();
						MorthalinRaidTools:GetZones();
					end
				elseif(prefix == "MRTHistory") then -- merge history
					for i = 1, #data do
						self:AddKillFromBroadcast(data[i], sender, true);
					end
					MorthalinRaidTools:GetZones();
					MorthalinRaidTools:UpdateHistory();
					MorthalinRaidTools:Print("History data was received by " .. sender ..  ".");
				elseif(prefix == "MRTAddHistPlayer") then -- add player or kill to history
					MorthalinRaidTools:AddPlayerToKillFromBroadcast(data, sender);
					MorthalinRaidTools:UpdateHistory();
				elseif(prefix == "MRTRemoveKill") then -- remove kill from history
					MorthalinRaidTools:RemoveKillFromBroadcast(data, sender);
					MorthalinRaidTools:UpdateHistory();
				elseif(prefix == "MRTAskForBench" and MorthalinRaidTools.db.global.mainBroadcaster) then
					MorthalinRaidTools:BroadcastBench(sender);
				elseif(prefix == "MRTAskForHist7" and MorthalinRaidTools.db.global.mainBroadcaster) then
					MorthalinRaidTools:BroadcastHistory7(sender);
				elseif(prefix == "MRTBuffSettings") then
					MorthalinRaidTools:BroadcastBuffSettings(data, sender);
				end
			elseif(prefix == "MRTHistory") then
				MorthalinRaidTools:Print("History data was sent to guild members.");
			end
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cffff0000" .. data);
		end
	end
end

function MorthalinRaidTools:BroadcastAddBench(player)
	if(MorthalinRaidTools.db.global.allowBroadcast) then
		local serialized = self:Serialize(player);
		self:SendCommMessage("MRTAddBench", serialized, "GUILD");
	end
end

function MorthalinRaidTools:AddBenchFromBroadcast(player, addedBy)
	local found = false;
	table.foreach(self.db.global.bench.players, function(index, value) 
		if(value.name == player.name) then 
			self:Print("|cffff0000Player |cff" .. value.color .. player.name .. "|cffff0000 is already on bench.");
			found = true;
		end 
	end)
	if(found == false) then
		table.insert(self.db.global.bench.players, player);
		self:Print(addedBy .. " added |cff" .. self.db.global.bench.players[#self.db.global.bench.players].color ..  player.name .. "|r on bench.");
		MorthalinRaidTools:UpdateBenchView();
	end
end

function MorthalinRaidTools:BroadcastRemoveBench(player)
	if(MorthalinRaidTools.db.global.allowBroadcast) then
		local serialized = self:Serialize(player);
		self:SendCommMessage("MRTRemoveBench", serialized, "GUILD");
	end
end

function MorthalinRaidTools:RemoveBenchFromBroadcast(player, removedBy)
	local found = false;
	table.foreach(self.db.global.bench.players, function(index, value) 
		if(value.name == player.name) then 
			table.remove(self.db.global.bench.players, index);
			self:Print("Player |cff" .. value.color .. player.name .. "|r was removed from bench by " .. removedBy .. ".");
			MorthalinRaidTools:UpdateBenchView();
			found = true;
		end 
	end)
	if(found == false) then
		self:Print("|cffff0000Player " .. player.name .. " is not on bench.");
	end
end

function MorthalinRaidTools:BroadcastBench(requester)
	if(MorthalinRaidTools.db.global.allowBroadcast) then
		local serialized = self:Serialize(self.db.global.bench.players);
		self:SendCommMessage("MRTFullBench", serialized, "GUILD");
	end
end

function MorthalinRaidTools:AskForBenchBroadcast()
	local serialized = self:Serialize("MRTAskForBench");
	self:SendCommMessage("MRTAskForBench", serialized, "GUILD");
end

function MorthalinRaidTools:AskForHistory7Broadcast()
	local serialized = self:Serialize("MRTAskForHist7");
	self:SendCommMessage("MRTAskForHist7", serialized, "GUILD");
end

function MorthalinRaidTools:BroadcastEmptyBench()
	if(MorthalinRaidTools.db.global.allowBroadcast) then
		local serialized = self:Serialize("true");
		self:SendCommMessage("MRTEmptyBench", serialized, "GUILD");
	end
end

function MorthalinRaidTools:BroadcastKill(kill)
	if(MorthalinRaidTools.db.global.allowBroadcast) then
		local serialized = self:Serialize(kill);
		self:SendCommMessage("MRTKill", serialized, "GUILD");
	end
end

function MorthalinRaidTools:BroadcastHistory()
	if(MorthalinRaidTools.db.global.allowBroadcast) then
		local serialized = MorthalinRaidTools:Serialize(self.db.global.history);
		local compressed = LibDeflate:CompressDeflate(serialized, {level = 9});
		local packet;
		if compressed then
			packet = LibDeflate:EncodeForWoWAddonChannel(compressed);
		end
		MorthalinRaidTools:Print("Sending history data...");
		MorthalinRaidTools:SendCommMessage("MRTHistory", packet, "GUILD");
	end
end

function MorthalinRaidTools:BroadcastHistory7(requester)
	local selected = {};
	local looping = true;
	local i = #self.db.global.history;
	while looping do
		if(self.db.global.history[i].killDate > (GetServerTime() - 604800) and i > 0) then
			table.insert(selected,self.db.global.history[i]);
		else
			looping = false;
		end
		i = i - 1;
	end
	
	local serialized = MorthalinRaidTools:Serialize(selected);
	local compressed = LibDeflate:CompressDeflate(serialized, {level = 9});
	local packet;
	if compressed then
		packet = LibDeflate:EncodeForWoWAddonChannel(compressed);
	end
	MorthalinRaidTools:Print("Sending history data for " .. requester);
	MorthalinRaidTools:SendCommMessage("MRTHistory", packet, "GUILD");
end

function MorthalinRaidTools:AddKillFromBroadcast(kill, sender, merge)
	local found = false;
	for i = 1, #self.db.global.history do
		if(self.db.global.history[i].bossID == kill.bossID and self.db.global.history[i].instanceID == kill.instanceID) then
			found = true;
			if(merge ~= nil) then
				for j = 1, #kill.members do
					if(kill.members[j].group == 0) then
						local found = false;
						for k = 1, #self.db.global.history[i].members do
							if(self.db.global.history[i].members[k].group == 0 and self.db.global.history[i].members[k].name == kill.members[j].name) then
								found = true;
							end
						end
						if(found == false) then
							table.insert(self.db.global.history[i].members, kill.members[j]);
						end
					end
				end
			end
		end
	end
	if(found == false) then
		table.insert(self.db.global.history, kill);
		if(merge == nil) then
			self:Print("Guild killed " .. kill.bossName .. " in " .. kill.zone .. ". Sent by " .. sender .. ".");
		end
	end
end

function MorthalinRaidTools:BroadcastHistoryAdd(kill)
	if(MorthalinRaidTools.db.global.allowBroadcast) then
		local serialized = self:Serialize(kill);
		self:SendCommMessage("MRTAddHistPlayer", serialized, "GUILD");
	end
end

function MorthalinRaidTools:AddPlayerToKillFromBroadcast(kill, sender)
	local index = 0;
	for i = 1, #self.db.global.history do
		if(self.db.global.history[i].bossID == kill.bossID and self.db.global.history[i].instanceID == kill.instanceID and self.db.global.history[i].resetDate == kill.resetDate) then
			index = i;
		end
	end
	
	if(index == 0) then
		table.insert(self.db.global.history, data);
		self:Print(sender .. " added missing " .. kill.bossName .. " kill from " .. date("%d.%m.%Y",kill.killDate) .. " to history.");
	elseif(#self.db.global.history[index].members <= #kill.members or self.db.global.allowBroadcastRemove) then
		self.db.global.history[index].members = kill.members;
		self:Print(sender .. " updated members on " .. kill.bossName .. "kill from " .. date("%d.%m.%Y",kill.killDate) .. ".");
	end
end

function MorthalinRaidTools:BroadcastHistoryRemove(kill)
	if(MorthalinRaidTools.db.global.allowBroadcast) then
		local serialized = self:Serialize(kill);
		self:SendCommMessage("MRTAddHistPlayer", serialized, "GUILD");
	end
end

function MorthalinRaidTools:RemoveKillFromBroadcast(kill, sender)
	local index = 0;
	for i = 1, #self.db.global.history do
		if(self.db.global.history[i].bossID == kill.bossID and self.db.global.history[i].instanceID == kill.instanceID) then
			index = i;
		end
	end
	
	if(index ~= 0 and self.db.global.allowBroadcastRemove) then
		table.remove(self.db.global.history, index);
		self:Print(sender .. " removed " .. kill.bossName .. "kill from " .. date("%d.%m.%Y",kill.killDate) .. ".");
	end
end

function MorthalinRaidTools:SendBuffSettings(target)
	local serialized = self:Serialize(self.db.global.buffs);
	self:SendCommMessage("MRTBuffSettings", serialized, "WHISPER", target);
	MorthalinRaidTools:Print("Buff settings sent to " .. target .. ".");
end

function MorthalinRaidTools:BroadcastBuffSettings(data, sender)
	local buffs = {};
	for i = 1, #self.db.global.buffs do
		if(buffs[self.db.global.buffs[i].name] == nil) then
			buffs[self.db.global.buffs[i].name] = self.db.global.buffs[i].name;
		end
	end
	for i = 1, #data do
		if(buffs[data[i].name] == nil) then
			table.insert(self.db.global.buffs, data[i]);
		end
	end
	self:Print(sender .. " sent you buffs settings.");
end
