--COLORS OF CLASSES
local classColors = {
	["Druid"] = { r = 1, g = 0.49, b = 0.04, hex = "FF7D0A" },
	["Hunter"] = {  r = 0.67, g = 0.83, b = 0.45, hex = "ABD473" },
	["Mage"] = { r = 0.25, g = 0.78, b = 0.92, hex = "40C7EB" },
	["Priest"] = { r = 1, g = 1, b = 1, hex = "FFFFFF" },
	["Rogue"] = { r = 1, g = 0.96, b = 0.41, hex = "FFF569" },
	["Shaman"] = { r = 0.96, g = 0.55, b = 0.73, hex = "F58CBA" },
	["Paladin"] = { r = 0.96, g = 0.55, b = 0.73, hex = "F58CBA" },
	["Warlock"] = { r = 0.53, g = 0.53, b = 0.93, hex = "8787ED" },
	["Warrior"] = { r = 0.78, g = 0.61, b = 0.43, hex = "C79C6E" }
}

function MorthalinRaidTools:GetClassColor(class)
	return classColors[class].hex;
end

function MorthalinRaidTools:AllowChat()
	if self.db.global.acceptBench then
		-- disable chat functions
		self.db.global.acceptBench = false;
	    MorthalinRaidTools:UnregisterEvent("CHAT_MSG_RAID");
		MorthalinRaidTools:UnregisterEvent("CHAT_MSG_RAID_LEADER");
		MorthalinRaidTools:UnregisterEvent("CHAT_MSG_GUILD");
		MorthalinRaidTools:UnregisterEvent("CHAT_MSG_WHISPER");
		self:Print("|cffff0000Addon stopped accepting raid members.")
	else
		-- enable chat functions
		self.db.global.acceptBench = true;
		MorthalinRaidTools:RegisterEvent("CHAT_MSG_RAID", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
		MorthalinRaidTools:RegisterEvent("CHAT_MSG_RAID_LEADER", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
	    MorthalinRaidTools:RegisterEvent("CHAT_MSG_GUILD", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
	    MorthalinRaidTools:RegisterEvent("CHAT_MSG_WHISPER", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
		self:Print("|cff00ff00Addon accepting raid members.")
	end
end

-- Function on command /mg bench "player name"
-- =========================
function MorthalinRaidTools:AddOnBench(playerName, delegate)
	local players = self.db.global.bench.players;
	local added = false;
	local found = false;
	playerName = string.upper(string.sub(playerName, 1, 1))..string.lower(string.sub(playerName, 2));
	table.foreach(players, function(index, value) 
		if(value.name == playerName) then 
			self:Print("|cffff0000Player |cff" .. value.color .. playerName .. "|cffff0000 is already in list.");
			found = true;
		end 
	end)
	
	if(found == false) then
		local numTotalMembers = GetNumGuildMembers();
		local realmName = GetRealmName();
		local playerTemp = {
			name = {},
			class = {},
			color = {},
			group = {}
		}
		
		for i=1, numTotalMembers do
			local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
			if(name == playerName .. "-" .. realmName) then
				table.insert(players, playerTemp);
				players[#players].name = playerName;
				players[#players].class = class;
				players[#players].color = classColors[class].hex;
				players[#players].group = 0;
				self:Print("|cff" .. players[#players].color .. playerName .. "|r added on bench.");
				if(delegate ~= nil) then
					SendChatMessage("You are on bench with nick " .. playerName .. ".", "WHISPER", "Common", delegate);
				end
				added = true;
				MorthalinRaidTools:BroadcastAddBench(players[#players]);
			end
		end
		
		if(added == false) then
			self:Print("|cffff0000Player " .. playerName .. " is not in guild or not exist.");
		end
		
		self:UpdateBenchView();
	end
end

function MorthalinRaidTools:CheckBenchInRaid()
	for i = 1, 40 do
		local name = GetRaidRosterInfo(i);
		if(name ~= nil) then
			for j = 1, #self.db.global.bench.players do
				if(self.db.global.bench.players[j].name == name) then
					self:Print("Player |cff" .. self.db.global.bench.players[j].color .. name .. "|r was moved to raid.");
					table.remove(self.db.global.bench.players, j);
					self:UpdateBenchView();
					return;
				end
			end
		end
	end
end

-- Function on command /mg rm "player name"
-- =========================
function MorthalinRaidTools:RemoveFromBench(playerName)
	local players = self.db.global.bench.players;
	local removed = false;
	playerName = string.upper(string.sub(playerName, 1, 1))..string.lower(string.sub(playerName, 2));
	table.foreach(players, function(index, value) 
		if(value.name == playerName) then 
			MorthalinRaidTools:BroadcastRemoveBench(value);
			table.remove(players, index);
			removed = true;
			self:Print("Player |cff" .. value.color .. playerName .. "|r was removed from bench.");
		end 
	end)
	if(removed == false) then
		self:Print("|cffff0000Player " .. playerName .. " is not on bench.");
	end
	
	self:UpdateBenchView();
end

-- Function on command /mg list
-- =========================
function MorthalinRaidTools:GetList()
	if(#self.db.global.bench.players ~= 0) then
		self:Print("Players on bench:");
		for index, player in pairs(self.db.global.bench.players) do
			self:Print("|cff" .. player.color .. player.name);
		end
	else
		self:Print("|cffff0000List is empty!");
	end
end

-- Function on command /mg history
-- =========================
function MorthalinRaidTools:ShowHistory()
	self:Print("Kills:");
	for index, player in pairs(self.db.global.history) do
		self:Print(date("%d.%m.%Y %H:%M:%S",player.killDate) .. " - " .. player.bossName);
	end
end

-- Function on command /mg reset
-- =========================
function MorthalinRaidTools:ResetList(broadcast)
	if(broadcast == nil) then
		self:BroadcastEmptyBench();
	end
	self.db.global.bench.players = {};
	self:Print("Bench is cleared.");
	self:UpdateBenchView();
end

function MorthalinRaidTools:AddBossKill(encounterID, encounterName)
	local instID = 1;
	if(encounterName == "Onyxia") then
		instID = tonumber(date("%d%m%y%H",GetServerTime()));
	end
	local resetSec;
	local encounterZone = MorthalinRaidTools.db.global.encounterList[encounterName].instance;
	RequestRaidInfo();
	local savedInstances = GetNumSavedInstances();
	for i = 1, savedInstances do
		local name, id, reset = GetSavedInstanceInfo(i);
		if(encounterZone == name) then
			instID = id;
			resetSec = reset;
		end
	end
	local guildMembers = 0;
	local raidMembers = 0;
	
	MorthalinRaidTools:Print(encounterName .. " killed at " .. encounterZone .. "(" .. instID .. ")");
	
	local timesheet = {
		bossID = encounterID,
		bossName = encounterName,
		killDate = GetServerTime(),
		zone = encounterZone,
		instanceID = instID,
		resetDate = resetSec ~= nil and date("%d.%m.%Y",GetServerTime() + resetSec) or MorthalinRaidTools:SetResetDate(GetServerTime(), encounterZone),
		members = {},
	}
	
	MorthalinRaidTools:SetResetGlobal(timesheet.resetDate, timesheet.zone);
	
	for i=1, 40 do
		local playerTemp = {
			name = {},
			class = {},
			color = {},
			group = {},
			buffs = {}
		}
		local name, rank, subgroup, level, class, fileName, zone, online = GetRaidRosterInfo(i);
		
		if(name ~= nil) then
			table.insert(timesheet.members, playerTemp);
			timesheet.members[#timesheet.members].name = name;
			timesheet.members[#timesheet.members].class = class;
			timesheet.members[#timesheet.members].color = classColors[class].hex;
			timesheet.members[#timesheet.members].group = subgroup;
			if(MorthalinRaidTools.db.global.members[name] ~= nil) then
				timesheet.members[#timesheet.members].buffs = MorthalinRaidTools.db.global.members[name];
			end
			if MorthalinRaidTools:IsGuildMember(name) then
				guildMembers = guildMembers + 1;
			end
			raidMembers = raidMembers + 1;
		else
			break;
		end
	end
	
	self:CheckBenchInRaid();
	
	if(#self.db.global.bench.players ~= 0) then
		for index, player in pairs(self.db.global.bench.players) do
			local playerTemp = {
				name = {},
				class = {},
				color = {},
				group = {}
			}
			table.insert(timesheet.members, playerTemp);
			timesheet.members[#timesheet.members].name = player.name;
			timesheet.members[#timesheet.members].class = player.class;
			timesheet.members[#timesheet.members].color = player.color;
			timesheet.members[#timesheet.members].group = player.group;
		end
	end
	
	if(self.db.global.history[#self.db.global.history].resetDate == timesheet.resetDate and self.db.global.history[#self.db.global.history].instanceID == 1) then
		self.db.global.history[#self.db.global.history].instanceID = timesheet.instanceID;
	end
	
	if (self.db.global.allowOnlyGuildKills == false or (guildMembers * 100 / raidMembers) >= 50) then
		table.insert(self.db.global.history, timesheet);
	end
	if (self.db.global.allowOnlyGuildBroadcast == false or (guildMembers * 100 / raidMembers) >= 50) then 
		self:BroadcastKill(self.db.global.history[#self.db.global.history]);
	end
end

function MorthalinRaidTools:ResetHistory()
	self.db.global.history = {};
end

function MorthalinRaidTools:SetZones()
	local rows = self.db.global.history;
	for i=1, #rows do
		if(rows[i].zone == nil) then
			rows[i].zone = self.db.global.encounterList[rows[i].bossName].instance;
		end
	end
end

function MorthalinRaidTools:Reload()
	ReloadUI();
end

function MorthalinRaidTools:Round(num, numDecimalPlaces)
  local mult = 10^(numDecimalPlaces or 0)
  return math.floor(num * mult + 0.5) / mult
end

function MorthalinRaidTools:GetPercentageWithColor(value, maxValue)
	if(maxValue ~= 0) then
		local percentage = self:Round(value * 100 / maxValue, 0);
		if(percentage < 10) then
			return "|cffff0000" .. percentage .. "%";
		elseif(percentage < 20) then
			return "|cffff2a00" .. percentage .. "%";
		elseif(percentage < 30) then
			return "|cffff5500" .. percentage .. "%";
		elseif(percentage < 40) then
			return "|cffff7f00" .. percentage .. "%";
		elseif(percentage < 50) then
			return "|cffffa800" .. percentage .. "%";
		elseif(percentage < 60) then
			return "|cffa8ff00" .. percentage .. "%";
		elseif(percentage < 70) then
			return "|cff7fff00" .. percentage .. "%";
		elseif(percentage < 80) then
			return "|cff55ff00" .. percentage .. "%";
		elseif(percentage < 90) then
			return "|cff2aff00" .. percentage .. "%";
		else
			return "|cff00ff00" .. percentage .. "%";
		end
	else
		return "|cffff0000No kills";
	end
end

function MorthalinRaidTools:GetRatioWithColor(value, maxValue, secondary)
	local percentage = self:Round(value * 100 / maxValue, 0);
	local parentesis = "";
	if(secondary ~= nil and secondary ~= 0) then
		parentesis = "(" .. secondary .. ")";
	end
	if(percentage < 10) then
		return "|cffff0000" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 20) then
		return "|cffff2a00" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 30) then
		return "|cffff5500" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 40) then
		return "|cffff7f00" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 50) then
		return "|cffffa800" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 60) then
		return "|cffa8ff00" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 70) then
		return "|cff7fff00" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 80) then
		return "|cff55ff00" .. parentesis .. value .. "/" .. maxValue;
	elseif(percentage < 90) then
		return "|cff2aff00" .. parentesis .. value .. "/" .. maxValue;
	else
		return "|cff00ff00" .. parentesis .. value .. "/" .. maxValue;
	end
end

function MorthalinRaidTools:ConvertStringIndex(tbl)
	local temp = {};
	for index, data in pairs(tbl) do
		table.insert(temp, data);
		temp[#temp].index = index;
	end
	return temp;
end

function MorthalinRaidTools:SetResetGlobal(reset, instance)
	local dayP, monthP, yearP = reset:match('^(%d%d).(%d%d).(%d%d%d%d)$');
	local resetInt = time{year=yearP, month=monthP, day=dayP, hour=9};
	if(instance == "Molten Core" or instance == "Blackwing Lair" or instance == "Ahn'Qiraj Temple") then
		MorthalinRaidTools.db.global.resets["MC"] = resetInt;
	elseif(instance == "Onyxia's Lair") then
		MorthalinRaidTools.db.global.resets["Onyx"] = resetInt;
	elseif(instance == "Zul'Gurub" or instance == "Ruins of Ahn'Qiraj") then
		MorthalinRaidTools.db.global.resets["ZG"] = resetInt;
	end
end

function MorthalinRaidTools:SetResetDate(killDate, instance)
	local looping = true;
	local reset;
	local step;
	if(instance == "Molten Core" or instance == "Blackwing Lair" or instance == "Ahn'Qiraj Temple") then
		reset = MorthalinRaidTools.db.global.resets["MC"];
		step = 604800;
	elseif(instance == "Onyxia's Lair") then
		reset = MorthalinRaidTools.db.global.resets["Onyx"];
		step = 432000;
	elseif(instance == "Zul'Gurub" or instance == "Ruins of Ahn'Qiraj") then
		reset = MorthalinRaidTools.db.global.resets["ZG"];
		step = 259200;
	end
	while looping do
		if(killDate < reset) then
			looping = false;
		else
			reset = reset + step;
		end
	end
	return date("%d.%m.%Y", reset);
end

function MorthalinRaidTools:OnRaidCreated()
	if(self.db.global.checkBenchAuto and self.db.global.acceptBench == false) then
		self:AllowChat();
	end
end

function MorthalinRaidTools:OnRaidLeave()
	if(self.db.global.checkBenchAuto and self.db.global.acceptBench == true) then
		self:AllowChat();
	end
	if(self.db.global.autoBenchEmpty and #self.db.global.bench.players ~= 0) then
		self:ConstructConfirm("Empty Bench", "Do you want empty bench?", "Empty Bench", 6);
	end
end

function MorthalinRaidTools:FindPlayers(template)
	local temp;
	if(string.len(template) > 1) then
		temp = string.upper(string.sub(template, 1, 1))..string.lower(string.sub(template, 2));
	elseif(string.len(template) == 1) then
		temp = string.upper(string.sub(template, 1, 1));
	end
	local matches = {};
	local numTotalMembers = GetNumGuildMembers();
	for i = 1, numTotalMembers do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
		if(temp == string.sub(name, 1, string.len(temp))) then
			table.insert(matches, {text = "|cff" .. classColors[class].hex .. string.sub(name, 1, (-1) * (string.len(GetRealmName()) + 2)), name = string.sub(name, 1, (-1) * (string.len(GetRealmName()) + 2)), rank = rankIndex});
		end
	end
	
	local function sortMatches(a, b)
		if(a.rank == b.rank) then
			return a.name < b.name;
		else
			return a.rank < b.rank;
		end
	end
	table.sort(matches, sortMatches);
	
	return matches;
end

function MorthalinRaidTools:GetGuildRanks()
	local ranks = {}
	for i = 1, GuildControlGetNumRanks() do
		table.insert(ranks, GuildControlGetRankName(i));
	end
	return ranks;
end

function MorthalinRaidTools:GetPlayerInfo(template)
	local temp;
	if(string.len(template) > 1) then
		temp = string.upper(string.sub(template, 1, 1))..string.lower(string.sub(template, 2));
	elseif(string.len(template) == 1) then
		temp = string.upper(string.sub(template, 1, 1));
	end
	local numTotalMembers = GetNumGuildMembers();
	for i = 1, numTotalMembers do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
		if(temp == string.sub(name, 1, string.len(temp))) then
			return {name = name, class = class, rank = rank, rankIndex = rankIndex};
		end
	end
end

function MorthalinRaidTools:IsGuildMember(playerName)
	local found = false;
	local i = 1;
	local numTotalMembers = GetNumGuildMembers();
	local realmName = GetRealmName();
	while found == false do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
		if(name == playerName .. "-" .. realmName) then
			found = true;
		end
		if(i < numTotalMembers) then
			i = i + 1;
		else
			return found;
		end
	end
	return found;
end

function MorthalinRaidTools:ClearGambleStatistics()
	MorthalinRaidTools.db.global.gambleStatistics = {};
	MorthalinRaidTools:Print("Gamble statistics was cleared.");
end

function MorthalinRaidTools:ChechBuffs()
	MorthalinRaidTools.db.global.members = {};
	for i = 1, 40 do
		local name, rank, subgroup, level, class, fileName, zone, online = GetRaidRosterInfo(i);
		if(online) then
			local playerName, _ = UnitName(name);
			if(MorthalinRaidTools.db.global.members[playerName] == nil) then
				MorthalinRaidTools.db.global.members[playerName] = {};
			end
			local buffExist = true;
			local index = 1;
			while(buffExist) do
				local buffName, icon, count, debuffType, duration, expirationTime, source, isStealable, nameplateShowPersonal, spellId = UnitAura(playerName, index);
				if(buffName ~= nil) then
					if(MorthalinRaidTools.db.global.members[playerName][buffName] == nil) then
						MorthalinRaidTools.db.global.members[playerName][buffName] = {};
						table.insert(MorthalinRaidTools.db.global.members[playerName][buffName], {icon = icon, spellId = spellId});
					else
						table.insert(MorthalinRaidTools.db.global.members[playerName][buffName], {icon = icon, spellId = spellId});
					end
					index = index + 1;
				else
					buffExist = false;
				end
			end
			MorthalinRaidTools:RemoveExpiredEnchants();
			if(MorthalinRaidTools.db.global.enchants[playerName] ~= nil) then
				local _, link = GetItemInfo(MorthalinRaidTools.db.global.enchants[playerName].name);
				local link = link:sub(18);
				link = link:sub(1, link:find(":") - 1);
				table.insert(MorthalinRaidTools.db.global.members[playerName][name], {icon = GetItemIcon(link), spellId = link});
			end
		end
	end
end

function MorthalinRaidTools:RemoveExpiredEnchants()
	for caster, data in pairs(MorthalinRaidTools.db.global.enchants) do
		if(GetServerTime() > data.expTime) then
			MorthalinRaidTools.db.global.enchants[caster] = nil;
		end
	end
end

function MorthalinRaidTools:ReconstructBuffs()
	for i = 1, #self.db.global.history do
		for j = 1, #self.db.global.history[i].members do
			if(self.db.global.history[i].members[j].buffs ~= nil) then
				local buffs = self.db.global.history[i].members[j].buffs;
				for name, icon in pairs(buffs) do
					if(type(icon) == "number") then
						local ic = icon;
						self.db.global.history[i].members[j].buffs[name] = {};
						table.insert(self.db.global.history[i].members[j].buffs[name], {icon = ic, spellId = 0});
					end
				end
			end
		end
	end
end

function MorthalinRaidTools:Test()
	--for i = 1, #self.db.global.history do
		--for i = #self.db.global.history - 30, #self.db.global.history do
			--if(self.db.global.history[i].bossID == self.db.global.history[j].bossID and self.db.global.history[i].instanceID == self.db.global.history[j].instanceID and i ~= j and self.db.global.history[i].instanceID ~= 1) then
				--self:Print(self.db.global.history[i].bossName .. " killed at " .. date("%d.%m.%Y %H:%M:%S",self.db.global.history[i].killDate) .. " ID: " .. self.db.global.history[i].instanceID);
			--end
		--end
	--end
	
	
	local slotId = GetInventorySlotInfo("MainHandSlot")
	local link = GetInventoryItemLink("raid1", slotId)
	local itemId, enchantId, gem1, gem2, gem3, gem4 = link:match("item:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+)")
	DEFAULT_CHAT_FRAME:AddMessage(link);
	
	--[[for name, value in pairs(MorthalinRaidTools.db.global.gambleStatistics) do
		MorthalinRaidTools:Print(name .. " have " .. value);
	end--]]
	--[[if(self.ui.iconTable ~= nil) then
		self.ui.iconTable:Hide();
	end
	RequestRaidInfo();
	local buffExist = true;
	local index = 1;
	local temp = {};
	local playerName, _ = UnitName("target");
	temp[playerName] = {}
	while(buffExist) do
		local name, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId  = UnitBuff("target", index);
		if(name ~= nil) then
			temp[playerName][name] = rank;
			index = index + 1;
			DEFAULT_CHAT_FRAME:AddMessage("name: "..name.. " ,icon:" .. icon.. " ,count:" .. tostring(count).. " ,debuffType:" .. debuffType.. " ,duration:" .. duration.. " ,expirationTime:" .. tostring(expirationTime).. " ,unitCaster:" .. tostring(unitCaster).. " ,isStealable:" .. tostring(isStealable).. " ,shouldConsolidate:" .. tostring(shouldConsolidate).. " ,spellId:" .. tostring(spellId));
		else
			buffExist = false;
		end
	end
	for i = 1, 39 do
		temp["tester " .. i] = temp[playerName];
	end--]]
	--MorthalinRaidTools:ConstructBuffsSettings();
	--MorthalinRaidTools:ChechBuffs();
	--MorthalinRaidTools:ConstructIconTable(self.ui.menuFrame.frame, temp);
    --DEFAULT_CHAT_FRAME:AddMessage(tostring(tonumber(GetAddOnMetadata("MorthalinRaidTools", "Version"))));
end
