local AceGUI = LibStub("AceGUI-3.0")

local killControl = nil;

-- MiniMap Icon Definition
-- =========================
function MorthalinRaidTools:ConstructMiniMapIcon() 
	LibStub("LibDBIcon-1.0"):Register("MorthalinRaidTools", LibStub("LibDataBroker-1.1"):NewDataObject("MorthalinRaidTools",
	{
		type = "data source",
		text = addonName,
		icon = "Interface\\AddOns\\MorthalinRaidTools\\media\\icon",
		OnClick = function(...) self:ToggleMenu() end,
		OnTooltipShow = function(tooltip)
			tooltip:AddLine("Morthalin Raid Tools",1,1,1);
			tooltip:Show();
		end
	}), self.db.char.minimap);
end

-- Menu toggle action
-- =========================
function MorthalinRaidTools:ToggleMenu()
	if MorthalinRaidTools.ui.menuFrame:IsVisible() then
		MorthalinRaidTools.ui.menuFrame:Hide();
		if(MorthalinRaidTools.ui.historyFrame:IsVisible()) then
			MorthalinRaidTools.ui.historyFrame:Hide();
		end
		if MorthalinRaidTools.ui.benchFrame:IsVisible() then
			MorthalinRaidTools.ui.benchFrame:Hide();
		end
	else
		MorthalinRaidTools.ui.menuFrame:Show();
	end
end

function MorthalinRaidTools:ConstructUI()
	MorthalinRaidTools.ui = {};
	MorthalinRaidTools:ConstructGamble();
	MorthalinRaidTools:ConstructHistorySettings();
	MorthalinRaidTools:ConstructHistoryFrame();
	MorthalinRaidTools:ConstructBenchFrame();
	MorthalinRaidTools:ConstructStatisticsFrame();
	MorthalinRaidTools:ConstructMenuFrame();
end

function MorthalinRaidTools:ConstructMenuFrame()
	self.ui.menuFrame = AceGUI:Create("Frame");
	self.ui.menuFrame:SetTitle("Menu");
	self.ui.menuFrame:SetStatusText("Main menu v"  .. self.db.global.version);
	self.ui.menuFrame:SetCallback("OnClose", function(widget)
		if(MorthalinRaidTools.ui.historyFrame:IsVisible()) then 
			MorthalinRaidTools.ui.historyFrame:Hide(); 
		end
		if(MorthalinRaidTools.ui.benchFrame:IsVisible()) then 
			MorthalinRaidTools.ui.benchFrame:Hide(); 
		end
		if(MorthalinRaidTools.ui.statisticsFrame:IsVisible()) then 
			MorthalinRaidTools.ui.statisticsFrame:Hide(); 
		end
		MorthalinRaidTools:HideHistorySettings();
	end);
	self.ui.menuFrame:SetLayout("Flow");
	self.ui.menuFrame:SetWidth(400);
	if(self.debug) then
		self.ui.menuFrame:SetHeight(190);
	else
		self.ui.menuFrame:SetHeight(160);
	end
	self.ui.menuFrame:EnableResize(false);
	self.ui.menuFrame.frame:EnableMouse(true);
	self.ui.menuFrame.frame:SetUserPlaced(true);
	_G["MyGlobalFrameName"] = self.ui.menuFrame.frame;
    tinsert(UISpecialFrames, "MyGlobalFrameName");
	
	
	local raidGroup = AceGUI:Create("SimpleGroup");
	raidGroup:SetWidth(180);
	raidGroup:SetLayout("Flow");
	self.ui.menuFrame:AddChild(raidGroup);
	
	local btnBench = AceGUI:Create("Button");
	btnBench:SetText("Bench");
	btnBench:SetFullWidth(true);
	btnBench:SetCallback("OnClick", function() self:ToggleBench(); end);
	btnBench:SetCallback("OnEnter", function() self:ShowBench(btnBench.frame) end);
	btnBench:SetCallback("OnLeave", function() self:HideTooltip() end);
	raidGroup:AddChild(btnBench);
	
	local btnGamble = AceGUI:Create("Button");
	btnGamble:SetText("Gamble");
	btnGamble:SetFullWidth(true);
	btnGamble:SetCallback("OnClick", function() self:ShowGamble(); end);
	raidGroup:AddChild(btnGamble);
	
	local btnOptions = AceGUI:Create("Button");
	btnOptions:SetText("Options");
	btnOptions:SetFullWidth(true);
	btnOptions:SetCallback("OnClick", function() InterfaceOptionsFrame_OpenToCategory("Morthalin Raid Tools"); MorthalinRaidTools:ToggleMenu(); end);
	raidGroup:AddChild(btnOptions);
	
	
	local statisticsGroup = AceGUI:Create("SimpleGroup");
	statisticsGroup:SetWidth(180);
	statisticsGroup:SetLayout("Flow");
	self.ui.menuFrame:AddChild(statisticsGroup);
	
	local btnHistory = AceGUI:Create("Button");
	btnHistory:SetText("History");
	btnHistory:SetFullWidth(true);
	btnHistory:SetCallback("OnClick", function() self:ToggleHistory(); end);
	statisticsGroup:AddChild(btnHistory);
	
	local btnStatistics = AceGUI:Create("Button");
	btnStatistics:SetText("Statistics");
	btnStatistics:SetFullWidth(true);
	btnStatistics:SetCallback("OnClick", function() self:ToggleStatistics(); end);
	statisticsGroup:AddChild(btnStatistics);
	
	local btnStatistics = AceGUI:Create("Button");
	btnStatistics:SetText("History Settings");
	btnStatistics:SetFullWidth(true);
	btnStatistics:SetCallback("OnClick", function() self:ShowHistorySettings() end);
	statisticsGroup:AddChild(btnStatistics);
	
	
	
	if(self.debug) then
		local testHeader = AceGUI:Create("SimpleGroup");
		testHeader:SetFullWidth(true);
		testHeader:SetLayout("Flow");
		self.ui.menuFrame:AddChild(testHeader);
		testHeader:ClearAllPoints();
		testHeader:SetPoint("BOTTOMLEFT", self.ui.menuFrame.frame, 0, 5);
		
		local btnTest = AceGUI:Create("Button");
		btnTest:SetText("Test");
		btnTest:SetWidth(120);
		btnTest:SetCallback("OnClick", function() self:Test(); end)
		testHeader:AddChild(btnTest);
		
		local btnReload = AceGUI:Create("Button");
		btnReload:SetText("Reload");
		btnReload:SetWidth(120);
		btnReload:SetCallback("OnClick", function() self:Reload(); end)
		testHeader:AddChild(btnReload);
	end
	self.ui.menuFrame:Hide();
end

function MorthalinRaidTools:HideTooltip()
	if(GameTooltip:IsVisible()) then
		GameTooltip:Hide();
	end
end

function MorthalinRaidTools:ConstructConfirm(title, text, status, typeOfConfirm)
	if(self.ui.confirmFrame ~= nil and self.ui.confirmFrame:IsVisible()) then
		self.ui.confirmFrame:Hide();
	end
	self.ui.confirmFrame = AceGUI:Create("Frame");
	self.ui.confirmFrame:SetTitle(title);
	self.ui.confirmFrame:SetStatusText(status);
	self.ui.confirmFrame:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end);
	self.ui.confirmFrame:SetLayout("Flow");
	self.ui.confirmFrame:SetWidth(350);
	self.ui.confirmFrame:SetHeight(115);
	self.ui.confirmFrame:EnableResize(false);
	
	local label = AceGUI:Create("Label");
	label:SetText(text);
	label:SetFullWidth(true);
	self.ui.confirmFrame:AddChild(label);
	
	local btnYes = AceGUI:Create("Button");
	btnYes:SetText("Yes");
	btnYes:SetWidth(150);
	btnYes:SetCallback("OnClick", function() 
		if(typeOfConfirm == 1) then
			self:RemovePlayerConfirmed();
			self.ui.confirmFrame:Hide();
		elseif(typeOfConfirm == 2) then
			self:RemoveBossConfirmed();
			self.ui.confirmFrame:Hide();
		elseif(typeOfConfirm == 3) then
			self:RemoveInstanceConfirmed();
			self.ui.confirmFrame:Hide();
		elseif(typeOfConfirm == 4) then
			self:AllowChat();
			self.ui.confirmFrame:Hide();
		elseif(typeOfConfirm == 5) then
			self:AllowKills();
			self.ui.confirmFrame:Hide();
		elseif(typeOfConfirm == 6) then
			self:ResetList();
			self.ui.confirmFrame:Hide();
			--[[if(self.db.global.acceptBench == true) then
				self:ConstructConfirm("Bench invites", "Bench invites are still active. Do you want to disable it?", "Bench invites", 7);
			else
				
			end--]]
		elseif(typeOfConfirm == 7) then
			self:AllowChat();
			self.ui.confirmFrame:Hide();
		elseif(typeOfConfirm == 8) then
			self:DeletePlayerComfirmed();
			self.ui.confirmFrame:Hide();
		end
	end)
	self.ui.confirmFrame:AddChild(btnYes);
	
	local btnNo = AceGUI:Create("Button");
	btnNo:SetText("No");
	btnNo:SetWidth(150);
	btnNo:SetCallback("OnClick", function() self.ui.confirmFrame:Hide(); end)
	self.ui.confirmFrame:AddChild(btnNo);
end
