local AceGUI = LibStub("AceGUI-3.0")

local scrollStatistics = nil;
local historyBtn = nil;
local uniqueDates = nil;
local historyInstance = nil;
local historyDate = nil;
local historyKill = nil;
local sortType = 0;
local alts = {};
local playerForRemove = {};
local startDate = nil;
local startDateText = "";
local stopDate = nil;
local stopDateText = "";

local statisticsPlayerData = {};
local allKills = 0;
local mcKills = 0;
local onyKills = 0;
local bwlKills = 0;
local zgKills = 0;
local aqKills = 0;
local raqKills = 0;
local mainRaidKills = 0;
local optRaidKills = 0;

--COLORS OF CLASSES
local classColors = {
	["Druid"] = { r = 1, g = 0.49, b = 0.04, hex = "FF7D0A" },
	["Hunter"] = {  r = 0.67, g = 0.83, b = 0.45, hex = "ABD473" },
	["Mage"] = { r = 0.25, g = 0.78, b = 0.92, hex = "40C7EB" },
	["Priest"] = { r = 1, g = 1, b = 1, hex = "FFFFFF" },
	["Rogue"] = { r = 1, g = 0.96, b = 0.41, hex = "FFF569" },
	["Shaman"] = { r = 0.96, g = 0.55, b = 0.73, hex = "F58CBA" },
	["Paladin"] = { r = 0.96, g = 0.55, b = 0.73, hex = "F58CBA" },
	["Warlock"] = { r = 0.53, g = 0.53, b = 0.93, hex = "8787ED" },
	["Warrior"] = { r = 0.78, g = 0.61, b = 0.43, hex = "C79C6E" }
}

-- History toggle action
-- =========================
function MorthalinRaidTools:ToggleStatistics()
	if MorthalinRaidTools.ui.statisticsFrame:IsVisible() then
		MorthalinRaidTools.ui.statisticsFrame:Hide();
	else
		MorthalinRaidTools.ui.statisticsFrame:Show();
		MorthalinRaidTools:LoadStatistics();
		self:UpdateStatisticsView(sortType);
	end
end

function MorthalinRaidTools:ConstructStatisticsFrame()
	self.ui.statisticsFrame = AceGUI:Create("Frame");
	self.ui.statisticsFrame:SetTitle("Player Statistics");
	self.ui.statisticsFrame:SetStatusText("");
	self.ui.statisticsFrame:SetLayout("Flow");
	self.ui.statisticsFrame:SetWidth(1100);
	self.ui.statisticsFrame:EnableResize(false);
	self.ui.statisticsFrame.frame:EnableMouse(true);
	self.ui.statisticsFrame.frame:SetUserPlaced(true);
	
	-- CONTROLS HEADER
	local controlsHeader = AceGUI:Create("SimpleGroup");
	controlsHeader:SetFullWidth(true);
	controlsHeader:SetLayout("Flow");
	self.ui.statisticsFrame:AddChild(controlsHeader);
	
	startDate = AceGUI:Create("EditBox");
	startDate:SetLabel("Start Date");
	startDate:SetWidth(100);
	startDate:SetCallback("OnEnterPressed", function(widget, event, text) 
		if(self:CheckDateString(text) == false) then 
			GameTooltip:SetOwner(startDate.frame,"ANCHOR_RIGHT");
			GameTooltip:AddLine("Wrong Date Format");
			GameTooltip:Show();
		else
			GameTooltip:Hide();
			startDateText = text;
			MorthalinRaidTools:LoadStatistics();
			self:UpdateStatisticsView(sortType);
		end
	end);
	controlsHeader:AddChild(startDate);
	
	stopDate = AceGUI:Create("EditBox");
	stopDate:SetLabel("Stop Date");
	stopDate:SetWidth(100);
	stopDate:SetCallback("OnEnterPressed", function(widget, event, text) 
		if(self:CheckDateString(text) == false) then 
			GameTooltip:SetOwner(stopDate.frame,"ANCHOR_RIGHT");
			GameTooltip:AddLine("Wrong Date Format");
			GameTooltip:Show();
		else
			GameTooltip:Hide();
			stopDateText = text;
			MorthalinRaidTools:LoadStatistics();
			self:UpdateStatisticsView(sortType);
		end
	end);
	controlsHeader:AddChild(stopDate);
	
	-- TABLE HEADER
	local tableHeader = AceGUI:Create("SimpleGroup");
	tableHeader:SetFullWidth(true);
	tableHeader:SetLayout("Flow");
	self.ui.statisticsFrame:AddChild(tableHeader);

	local historyBtn = AceGUI:Create("InteractiveLabel");
	historyBtn:SetWidth(150);
	historyBtn:SetText("Player");
	historyBtn.highlight:SetColorTexture(0.3, 0.3, 0.3, 0.5)
	historyBtn:SetCallback("OnClick", function() sortType = 0; self:UpdateStatisticsView(sortType); end);
	tableHeader:AddChild(historyBtn);
	
	historyBtn = AceGUI:Create("InteractiveLabel");
	historyBtn:SetWidth(100);
	historyBtn:SetText("Overall");
	historyBtn.highlight:SetColorTexture(0.3, 0.3, 0.3, 0.5)
	historyBtn:SetCallback("OnClick", function() sortType = 1; self:UpdateStatisticsView(sortType); end);
	tableHeader:AddChild(historyBtn);
	
	historyBtn = AceGUI:Create("InteractiveLabel");
	historyBtn:SetWidth(100);
	historyBtn:SetText("Main Raids");
	historyBtn.highlight:SetColorTexture(0.3, 0.3, 0.3, 0.5)
	historyBtn:SetCallback("OnClick", function() sortType = 6; self:UpdateStatisticsView(sortType); end);
	tableHeader:AddChild(historyBtn);
	
	historyBtn = AceGUI:Create("InteractiveLabel");
	historyBtn:SetWidth(100);
	historyBtn:SetText("Optional");
	historyBtn.highlight:SetColorTexture(0.3, 0.3, 0.3, 0.5)
	historyBtn:SetCallback("OnClick", function() sortType = 7; self:UpdateStatisticsView(sortType); end);
	tableHeader:AddChild(historyBtn);
	
	historyBtn = AceGUI:Create("InteractiveLabel");
	historyBtn:SetWidth(100);
	historyBtn:SetText("Molten Core");
	historyBtn.highlight:SetColorTexture(0.3, 0.3, 0.3, 0.5)
	historyBtn:SetCallback("OnClick", function() sortType = 2; self:UpdateStatisticsView(sortType); end);
	tableHeader:AddChild(historyBtn);
	
	historyBtn = AceGUI:Create("InteractiveLabel");
	historyBtn:SetWidth(100);
	historyBtn:SetText("Onyxia");
	historyBtn.highlight:SetColorTexture(0.3, 0.3, 0.3, 0.5)
	historyBtn:SetCallback("OnClick", function() sortType = 3; self:UpdateStatisticsView(sortType); end);
	tableHeader:AddChild(historyBtn);
	
	historyBtn = AceGUI:Create("InteractiveLabel");
	historyBtn:SetWidth(100);
	historyBtn:SetText("Blackwing Lair");
	historyBtn.highlight:SetColorTexture(0.3, 0.3, 0.3, 0.5)
	historyBtn:SetCallback("OnClick", function() sortType = 4; self:UpdateStatisticsView(sortType); end);
	tableHeader:AddChild(historyBtn);
	
	historyBtn = AceGUI:Create("InteractiveLabel");
	historyBtn:SetWidth(100);
	historyBtn:SetText("Zul'Gurub");
	historyBtn.highlight:SetColorTexture(0.3, 0.3, 0.3, 0.5)
	historyBtn:SetCallback("OnClick", function() sortType = 5; self:UpdateStatisticsView(sortType); end);
	tableHeader:AddChild(historyBtn);
	
	historyBtn = AceGUI:Create("InteractiveLabel");
	historyBtn:SetWidth(100);
	historyBtn:SetText("Ahn'Qiraj Temple");
	historyBtn.highlight:SetColorTexture(0.3, 0.3, 0.3, 0.5)
	historyBtn:SetCallback("OnClick", function() sortType = 8; self:UpdateStatisticsView(sortType); end);
	tableHeader:AddChild(historyBtn);
	
	historyBtn = AceGUI:Create("InteractiveLabel");
	historyBtn:SetWidth(100);
	historyBtn:SetText("Ruins of Ahn'Qiraj");
	historyBtn.highlight:SetColorTexture(0.3, 0.3, 0.3, 0.5)
	historyBtn:SetCallback("OnClick", function() sortType = 9; self:UpdateStatisticsView(sortType); end);
	tableHeader:AddChild(historyBtn);
	
	--TABLE DATA PART
	scrollcontainer = AceGUI:Create("SimpleGroup");
	scrollcontainer:SetFullWidth(true);
	scrollcontainer:SetHeight(360);
	scrollcontainer:SetLayout("Fill");
	self.ui.statisticsFrame:AddChild(scrollcontainer);
	scrollcontainer:ClearAllPoints();
	scrollcontainer.frame:SetPoint("TOP", tableHeader.frame, "BOTTOM", 0, -5);
	scrollcontainer.frame:SetPoint("BOTTOM", 0, 20);

	scrollStatistics = CreateFrame("ScrollFrame", nil, scrollcontainer.frame, "myScrollFrame");
	HybridScrollFrame_CreateButtons(scrollStatistics, "statisticsListItemTemplate");
	HybridScrollFrame_SetDoNotHideScrollBar(scrollStatistics, true);
	
	scrollStatistics.update = function() 
		self:UpdateStatisticsView(sortType);
	end
	self.ui.statisticsFrame:Hide();
end

function MorthalinRaidTools:CheckDateString(text)
	local day, month, year = text:match('(%d+).(%d+).(%d+)');
	
	if(day == nil or month == nil  or year == nil  or hour == nil  or minute == nil  or second == nil) then
		return false;
	end
	
	if(string.len(day) ~= 2) then
		day = "0" .. day;
	end
	if(string.len(month) ~= 2) then
		month = "0" .. month;
	end
	text = day .. "." .. month .. "." .. yearA;
	local dateInt = time{year=year, month=month, day=day};
	return text == date("%d.%m.%Y", dateInt);
end

function MorthalinRaidTools:LoadStatistics()
	statisticsPlayerData = {};
	allKills = 0;
	mcKills = 0;
	onyKills = 0;
	bwlKills = 0;
	zgKills = 0;
	aqKills = 0;
	raqKills = 0;
	mainRaidKills = 0;
	optRaidKills = 0;
	local minDate = -1;
	local maxDate = -1;
	local dayA, monthA, yearA = 0,0,0;
	if(string.len(startDateText) ~= 0) then
		dayA, monthA, yearA = startDateText:match('(%d+).(%d+).(%d+)');
	end
	local dateStartInt = 0;
	if(string.len(startDateText) ~= 0 and MorthalinRaidTools:CheckDateString(startDateText)) then
		dateStartInt = time{year=yearA, month=monthA, day=dayA};
	end
	local dayB, monthB, yearB = 0,0,0;
	if(string.len(stopDateText) ~= 0) then
		dayB, monthB, yearB = stopDateText:match('(%d+).(%d+).(%d+)');
	end
	local dateStopInt = GetServerTime();
	if(string.len(stopDateText) ~= 0 and MorthalinRaidTools:CheckDateString(stopDateText)) then
		dateStopInt = time{year=yearB, month=monthB, day=dayB};
	end
	
	local uniqueHistory = {};
	for i = 1, #self.db.global.history do
		if(dateStartInt <= self.db.global.history[i].killDate and dateStopInt >= self.db.global.history[i].killDate) then
			if(uniqueHistory[self.db.global.history[i].zone] == nil) then
				uniqueHistory[self.db.global.history[i].zone] = {resets = {}};
			end
			if(uniqueHistory[self.db.global.history[i].zone].resets[self.db.global.history[i].resetDate] == nil) then
				uniqueHistory[self.db.global.history[i].zone].resets[self.db.global.history[i].resetDate] = {bosses = {}};
			end
			if(uniqueHistory[self.db.global.history[i].zone].resets[self.db.global.history[i].resetDate].bosses[self.db.global.history[i].bossID] == nil) then
				uniqueHistory[self.db.global.history[i].zone].resets[self.db.global.history[i].resetDate].bosses[self.db.global.history[i].bossID] = true;
				if(self.db.global.history[i].zone == "Molten Core") then
					mcKills = mcKills + 1;
				elseif(self.db.global.history[i].zone == "Onyxia's Lair") then
					onyKills = onyKills + 1;
				elseif(self.db.global.history[i].zone == "Blackwing Lair") then
					bwlKills = bwlKills + 1;
				elseif(self.db.global.history[i].zone == "Zul'Gurub") then
					zgKills = zgKills + 1;
				elseif(self.db.global.history[i].zone == "Ahn'Qiraj Temple") then
					aqKills = aqKills + 1;
				elseif(self.db.global.history[i].zone == "Ruins of Ahn'Qiraj") then
					raqKills = raqKills + 1;
				end
				local found = false;
				for j = 1, #MorthalinRaidTools.db.global.mainRaids do
					if(MorthalinRaidTools.db.global.mainRaids[j].value and MorthalinRaidTools.db.global.mainRaids[j].zone == self.db.global.history[i].zone) then
						mainRaidKills = mainRaidKills + 1;
						found = true;
					end
				end
				if(found == false) then
					optRaidKills = optRaidKills + 1;
				end
				allKills = allKills + 1;
			end
			if(minDate == -1 or minDate > self.db.global.history[i].killDate) then
				minDate = self.db.global.history[i].killDate;
			end
			if(maxDate == -1 or maxDate < self.db.global.history[i].killDate) then
				maxDate = self.db.global.history[i].killDate;
			end
		end
	end
	
	if(string.len(startDate:GetText()) == 0) then
		startDate:SetText(date("%d.%m.%Y", minDate));
	end
	if(string.len(stopDate:GetText()) == 0) then
		stopDate:SetText(date("%d.%m.%Y", maxDate));
	end
	
	for i=1, GetNumGuildMembers() do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
		if(string.sub(officernote, 1, 4) == "Alt:") then
			if(alts[string.sub(name, 1, name:find("-") - 1)] == nil) then
				alts[string.sub(name, 1, name:find("-") - 1)] = string.sub(officernote, 5, -1);
			end
		end
	end
	
	local playerStatistics = {};
	for i = 1, #self.db.global.history do
		if(dateStartInt <= self.db.global.history[i].killDate and dateStopInt >= self.db.global.history[i].killDate) then
			for j = 1, #self.db.global.history[i].members do
				local tempName = {};
				if(alts[self.db.global.history[i].members[j].name] ~= nil) then
					tempName = alts[self.db.global.history[i].members[j].name];
				else
					tempName = self.db.global.history[i].members[j].name;
				end
				
				if(playerStatistics[tempName] == nil) then
					local playerStatisticsTemp = 
					{
						class = {},
						color = {},
						boss = {}
					}
					playerStatistics[tempName] = playerStatisticsTemp;
					if(alts[self.db.global.history[i].members[j].name] == nil) then
						playerStatistics[tempName].class = self.db.global.history[i].members[j].class;
						playerStatistics[tempName].color = self.db.global.history[i].members[j].color;
					else
						for i=1, GetNumGuildMembers() do
							local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
							if(tempName == string.sub(name, 1, name:find("-") - 1)) then
								playerStatistics[tempName].class = class;
								playerStatistics[tempName].color = classColors[class].hex;
							end
						end
					end
				end
				if(playerStatistics[tempName].boss[self.db.global.history[i].bossID] == nil) then
					local boss = {
						count = 1,
						name = self.db.global.history[i].bossName,
						zone = self.db.global.history[i].zone,
						dates = {}
					}
					playerStatistics[tempName].boss[self.db.global.history[i].bossID] = boss;
					playerStatistics[tempName].boss[self.db.global.history[i].bossID].dates[self.db.global.history[i].resetDate] = self.db.global.history[i].killDate;
				else
					if(playerStatistics[tempName].boss[self.db.global.history[i].bossID].dates[self.db.global.history[i].resetDate] == nil) then
						playerStatistics[tempName].boss[self.db.global.history[i].bossID].count = playerStatistics[tempName].boss[self.db.global.history[i].bossID].count + 1;
						playerStatistics[tempName].boss[self.db.global.history[i].bossID].dates[self.db.global.history[i].resetDate] = self.db.global.history[i].killDate;
					elseif(alts[self.db.global.history[i].members[j].name] ~= nil) then
						playerStatistics[tempName].boss[self.db.global.history[i].bossID].dates[self.db.global.history[i].resetDate] = self.db.global.history[i].killDate;
					end
				end
			end
		end
	end
	
	for index, player in pairs(playerStatistics) do
		local data = {
			name = index,
			class = player.class,
			color = player.color,
			count = 0,
			countMain = 0,
			countOpt = 0,
			boss = {},
			instances = {}
		}
		
		for id, bossData in pairs(player.boss) do
			local tempBossData = {
				encounterID = id,
				name = bossData.name,
				zone = bossData.zone,
				count = bossData.count
			}
			
			table.insert(data.boss, tempBossData);
			if(data.instances[bossData.zone] == nil) then
				data.instances[bossData.zone] = bossData.count;
			else
				data.instances[bossData.zone] = data.instances[bossData.zone] + bossData.count;
			end
			data.count = data.count + bossData.count;
			local found = false;
			for j = 1, #MorthalinRaidTools.db.global.mainRaids do
				if(MorthalinRaidTools.db.global.mainRaids[j].value and MorthalinRaidTools.db.global.mainRaids[j].zone == bossData.zone) then
					data.countMain = data.countMain + bossData.count;
					found = true;
				end
			end
			if(found == false) then
				data.countOpt = data.countOpt + bossData.count;
			end
		end
		local function sortBosses(a, b)
			if(a.zone == b.zone) then
				return a.name < b.name;
			else
				return a.zone < b.zone;
			end
		end
		table.sort(data.boss, sortBosses);
		table.insert(statisticsPlayerData, data);
	end
	
	local indexes = {};
	for i = 1, #statisticsPlayerData do
		if(statisticsPlayerData[(#statisticsPlayerData + 1 - i)].count == 0) then
			playerStatistics[statisticsPlayerData[(#statisticsPlayerData + 1 - i)].name] = nil;
			table.insert(indexes, (#statisticsPlayerData + 1 - i));
		end
	end
	
	for i = 1, #indexes do
		table.remove(statisticsPlayerData, indexes[i]);
	end
end

function MorthalinRaidTools:UpdateStatisticsView(sort)
	if(sort == nil) then
		sort = sortType;
	end
	local buttons = HybridScrollFrame_GetButtons(scrollStatistics);
	local offset = HybridScrollFrame_GetOffset(scrollStatistics);
	
	local function sortPlayers(a, b)
		if(sort == 0) then
			return a.name < b.name;
		elseif(sort == 1) then
			if(a.count == b.count) then
				return a.name < b.name;
			else
				return a.count > b.count;
			end
		elseif(sort == 2) then
			if((a.instances["Molten Core"] == nil and 0 or a.instances["Molten Core"]) == (b.instances["Molten Core"] == nil and 0 or b.instances["Molten Core"])) then
				return a.name < b.name;
			else
				return (a.instances["Molten Core"] == nil and 0 or a.instances["Molten Core"]) > (b.instances["Molten Core"] == nil and 0 or b.instances["Molten Core"])
			end
		elseif(sort == 3) then
			if((a.instances["Onyxia's Lair"] == nil and 0 or a.instances["Onyxia's Lair"]) == (b.instances["Onyxia's Lair"] == nil and 0 or b.instances["Onyxia's Lair"])) then
				return a.name < b.name;
			else
				return (a.instances["Onyxia's Lair"] == nil and 0 or a.instances["Onyxia's Lair"]) > (b.instances["Onyxia's Lair"] == nil and 0 or b.instances["Onyxia's Lair"])
			end
		elseif(sort == 4) then
			if((a.instances["Blackwing Lair"] == nil and 0 or a.instances["Blackwing Lair"]) == (b.instances["Blackwing Lair"] == nil and 0 or b.instances["Blackwing Lair"])) then
				return a.name < b.name;
			else
				return (a.instances["Blackwing Lair"] == nil and 0 or a.instances["Blackwing Lair"]) > (b.instances["Blackwing Lair"] == nil and 0 or b.instances["Blackwing Lair"])
			end
		elseif(sort == 5) then
			if((a.instances["Zul'Gurub"] == nil and 0 or a.instances["Zul'Gurub"]) == (b.instances["Zul'Gurub"] == nil and 0 or b.instances["Zul'Gurub"])) then
				return a.name < b.name;
			else
				return (a.instances["Zul'Gurub"] == nil and 0 or a.instances["Zul'Gurub"]) > (b.instances["Zul'Gurub"] == nil and 0 or b.instances["Zul'Gurub"])
			end
		elseif(sort == 6) then
			if(a.countMain == b.countMain) then
				return a.name < b.name;
			else
				return a.countMain > b.countMain;
			end
		elseif(sort == 7) then
			if(a.countOpt == b.countOpt) then
				return a.name < b.name;
			else
				return a.countOpt > b.countOpt;
			end
		elseif(sort == 8) then
			if((a.instances["Ahn'Qiraj Temple"] == nil and 0 or a.instances["Ahn'Qiraj Temple"]) == (b.instances["Ahn'Qiraj Temple"] == nil and 0 or b.instances["Ahn'Qiraj Temple"])) then
				return a.name < b.name;
			else
				return (a.instances["Ahn'Qiraj Temple"] == nil and 0 or a.instances["Ahn'Qiraj Temple"]) > (b.instances["Ahn'Qiraj Temple"] == nil and 0 or b.instances["Ahn'Qiraj Temple"])
			end
		elseif(sort == 9) then
			if((a.instances["Ruins of Ahn'Qiraj"] == nil and 0 or a.instances["Ruins of Ahn'Qiraj"]) == (b.instances["Ruins of Ahn'Qiraj"] == nil and 0 or b.instances["Ruins of Ahn'Qiraj"])) then
				return a.name < b.name;
			else
				return (a.instances["Ruins of Ahn'Qiraj"] == nil and 0 or a.instances["Ruins of Ahn'Qiraj"]) > (b.instances["Ruins of Ahn'Qiraj"] == nil and 0 or b.instances["Ruins of Ahn'Qiraj"])
			end
		end
	end
	table.sort(statisticsPlayerData, sortPlayers);
	
	
	for buttonIndex = 1, #buttons do
		local button = buttons[buttonIndex];
		local itemIndex = buttonIndex + offset;
		
		if (itemIndex <= #statisticsPlayerData) then
			button:SetID(itemIndex);
			button.Player:SetText("|cff" .. statisticsPlayerData[itemIndex].color .. statisticsPlayerData[itemIndex].name);
			button:SetScript('OnEnter', function() self:ShowAlts(button, statisticsPlayerData[itemIndex].name) end);
			button:SetScript('OnLeave', function() self:HideTooltip() end);
			button.Overall:SetText(self:GetPercentageWithColor(statisticsPlayerData[itemIndex].count, allKills));
			button.MainRaids:SetText(self:GetPercentageWithColor(statisticsPlayerData[itemIndex].countMain, mainRaidKills));
			button.OptRaids:SetText(self:GetPercentageWithColor(statisticsPlayerData[itemIndex].countOpt, optRaidKills));
			button.MoltenCore:SetText((mcKills == 0 and "No kills" or self:Round((statisticsPlayerData[itemIndex].instances["Molten Core"] == nil and 0 or statisticsPlayerData[itemIndex].instances["Molten Core"]) * 100 / mcKills, 0) .. "%"));
			button.Onyxia:SetText((onyKills == 0 and "No kills" or self:Round((statisticsPlayerData[itemIndex].instances["Onyxia's Lair"] == nil and 0 or statisticsPlayerData[itemIndex].instances["Onyxia's Lair"]) * 100 / onyKills, 0) .. "%"));
			button.BlackwingLair:SetText((bwlKills == 0 and "No kills" or self:Round((statisticsPlayerData[itemIndex].instances["Blackwing Lair"] == nil and 0 or statisticsPlayerData[itemIndex].instances["Blackwing Lair"]) * 100 / bwlKills, 0) .. "%"));
			button.ZulGurub:SetText((zgKills == 0 and "No kills" or self:Round((statisticsPlayerData[itemIndex].instances["Zul'Gurub"] == nil and 0 or statisticsPlayerData[itemIndex].instances["Zul'Gurub"]) * 100 / zgKills, 0) .. "%"));
			button.AhnQiraj:SetText((aqKills == 0 and "No kills" or self:Round((statisticsPlayerData[itemIndex].instances["Ahn'Qiraj Temple"] == nil and 0 or statisticsPlayerData[itemIndex].instances["Ahn'Qiraj Temple"]) * 100 / aqKills, 0) .. "%"));
			button.RuinsOfAhnQiraj:SetText((raqKills == 0 and "No kills" or self:Round((statisticsPlayerData[itemIndex].instances["Ruins of Ahn'Qiraj"] == nil and 0 or statisticsPlayerData[itemIndex].instances["Ruins of Ahn'Qiraj"]) * 100 / raqKills, 0) .. "%"));
			button:RegisterForClicks("MiddleButtonUp");
			button:SetScript('OnClick', function(self, button, down) if(button == "MiddleButton") then MorthalinRaidTools:DeletePlayerHistory(statisticsPlayerData[itemIndex]) end end)
			button:Show();
		else
			button:Hide();
		end
	end

	local buttonHeight = scrollStatistics.buttonHeight;
	local totalHeight = #statisticsPlayerData * buttonHeight;
	local shownHeight = #buttons * buttonHeight;

	HybridScrollFrame_Update(scrollStatistics, totalHeight, shownHeight);
end

function MorthalinRaidTools:ShowAlts(obj, name)
	GameTooltip:SetOwner(obj,"ANCHOR_LEFT");
	GameTooltip:AddLine("Alts:");
	for alt, main in pairs(alts) do
		if(main == name) then
			GameTooltip:AddLine(alt);
		end
	end
	GameTooltip:Show();
end

function MorthalinRaidTools:DeletePlayerHistory(player)
	playerForRemove = player;
	self:ConstructConfirm("Confirmation", "Removing |cff" .. playerForRemove.color .. playerForRemove.name .. "|r from history", "Player: " .. playerForRemove.name, 8);
end

function MorthalinRaidTools:DeletePlayerComfirmed()
	for i = 1, #self.db.global.history do 
	local count = #self.db.global.history[i].members;
		for j = 1, count do
			if(self.db.global.history[i].members[(count + 1 - j)].name == playerForRemove.name) then
				table.remove(self.db.global.history[i].members, (count + 1 - j));
			end
		end
	end
	self:UpdateStatisticsView();
	self:UpdateHistory();
end
